#
# MaMa, the Make Master -- a Tcl/Tk script for handling Makefile operations.
#
# Copyright (C) 1998 Asko Seeba.
#
# This file is considered as a part of MaMa package.
#
# The copyright holder disclaims all responsibility or liability with
# respect to its usage or its effect upon hardware or computer
# systems, and maintains copyright as set out in the "LICENCE"
# document which accompanies distributions of MaMa v1.0a0 and upwards.

#######################################################################
# Here comes the declaration of help system's initial settings.       #
#######################################################################

global HelpSectionFont HelpTextFont HelpTitleFont
set HelpSectionFont -b&h-lucida-bold-r-normal-*-0-180-72-72-p-100-iso8859-1
set HelpTextFont    -*-Helvetica-Medium-R-Normal--*-140-*-*-*-*-*-*
set HelpTitleFont   -*-Helvetica-Bold-R-Normal--*-180-*-*-*-*-*-*

#######################################################################
# Here comes the declaration of MaMa's help procedures.               #
#######################################################################

#
# ShowAbout --
# display an 'About' message.
#
proc ShowAbout {} {
    global MAMA_VER

    catch {destroy .about}
    toplevel .about
    wm title .about "About MaMa"
    wm iconname .about "About MaMa"
    positionWindow .about

    label .about.mama      -text "Make Master" \
	    -font -adobe-helvetica-bold-o-normal--0-180-100-100-p-124-iso8859-1
    label .about.version   -text "version $MAMA_VER"
    label .about.copyright -text "Copyright (C) 1998" \
	    -font -b&h-lucida-bold-r-normal-*-0-180-72-72-p-100-iso8859-1
    label .about.by        -text "by"
    label .about.asko      -text "Asko Seeba" \
	    -font -b&h-lucida-bold-i-normal-*-30-20-0-0-p-100-iso8859-1
    label .about.joka      -text "(alias Joka)" \
	    -font -b&h-lucida-bold-i-normal-*-30-20-0-0-p-100-iso8859-1
    button .about.ok        -text "OK" -command "destroy .about"
    pack .about.mama .about.version .about.copyright .about.by .about.asko \
	    .about.joka .about.ok -side top -fill x

    focus .about

    bind .about <Escape> {destroy .about}
    bind .about <Return> {destroy .about}
}

#
# ShowIntroduction --
# display 'Introduction' text.
#
proc ShowIntroduction {} {
    global HelpSectionFont HelpTextFont HelpTitleFont

    catch {destroy .intro}
    toplevel .intro
    wm title .intro "Introduction to MaMa"
    wm iconname .intro "Introduction"
    positionWindow .intro 100 10

    text      .intro.text -yscroll ".intro.scroll set" -wrap word -width 60 \
	    -height 30 -font $HelpTextFont -relief sunken -takefocus 0\
	    -setgrid 1
    scrollbar .intro.scroll -orient vertical -command ".intro.text yview" \
	    -highlightthickness 0
    button    .intro.close -text Close -command "destroy .intro"

    pack .intro.close  -side bottom -fill x
    pack .intro.scroll -side right -fill y
    pack .intro.text   -expand yes -fill both

    .intro.text tag configure title   -font $HelpTitleFont
    .intro.text tag configure section -font $HelpSectionFont

    .intro.text insert end "\n    Introduction to Make Master\n" title
    .intro.text insert end "\nMake Master (MaMa) provides a graphical front end for 'make' program. I do a lot of programming work at *NIX environment and for that I use always Makefiles. I started to write the interface from my spare time when I noticed myself all the time typing the same commands like 'make all', 'make foo', 'make bar', 'make depend', 'make all', 'make install' etc. So, I thought that why not to make it a little simpler and here it is. The application, that's basic aid is to get off of this annoying command-line 'make <target>' touch-typing and use some kind of hit-and-go interface instead. I hope You'll enjoy it too.\n"

    .intro.text insert end "\n\nMore Information about MaMa\n" section
    .intro.text insert end "\nHere are described briefly what else can be found in MaMa's help menu. See\n"
    .intro.text insert end "\n* 'Getting Started' for how to get quickly started with MaMa."
    .intro.text insert end "\n* 'Tutorial' for complete description of MaMa."
    .intro.text insert end "\n* 'Controls' for short description of MaMa menus, buttons and key controls."
    .intro.text insert end "\n* 'Done' to know what has been done in MaMa implementation."
    .intro.text insert end "\n* 'Ideas' to imagine a (not im)possible future of MaMa."
    .intro.text insert end "\n* 'Known Problems' to see if Your problem is already known problem."
    .intro.text insert end "\n* 'Support' for MaMa's support information.\n"

    .intro.text configure -state disabled
    focus .intro.scroll

    bind .intro       <Escape> {destroy .intro}
    bind .intro.close <Return> {destroy .intro}
}

#
# ShowGettingStarted --
# display 'Getting Started' text.
#
proc ShowGettingStarted {} {
    global HelpTextFont HelpTitleFont

    catch {destroy .getstart}
    toplevel .getstart
    wm title .getstart "Getting Started to MaMa"
    wm iconname .getstart "Getting Started"
    positionWindow .getstart 100 10

    text      .getstart.text -yscroll ".getstart.scroll set" -wrap word \
	    -width 60 -height 30 -font $HelpTextFont -relief sunken \
	    -takefocus 0 -setgrid 1
    scrollbar .getstart.scroll -orient vertical \
	    -command ".getstart.text yview" -highlightthickness 0
    button    .getstart.close -text Close -command "destroy .getstart"

    pack .getstart.close  -side bottom -fill x
    pack .getstart.scroll -side right -fill y
    pack .getstart.text   -expand yes -fill both

    .getstart.text tag configure title -font $HelpTitleFont
    .getstart.text insert end "\n    Getting Started  to Make Master\n" title
    .getstart.text insert end "\nHere is a step by step guide to get You quickly started with MaMa. Just follow these steps and You should understand what is going on.\n"
    .getstart.text insert end "\n1. Go to the directory where You have a makefile with several targets to make.\n"
    .getstart.text insert end "\n2. Run a MaMa.\n"
    .getstart.text insert end "\n3. On the right side You'll see a button box and an 'Add' button in it. Press the 'Add' button. A new window appears then.\n"
    .getstart.text insert end "\n4. Specify a makefile in that window. You can browse a directory for searching makefile by pressing 'Browse' button.\n"
    .getstart.text insert end "\n5. Specify an existing target in that makefile.\n"
    .getstart.text insert end "\n6. Add a comment line. That should describe with few words the nature of this target in the makefile.\n"
    .getstart.text insert end "\n7. Press an 'OK' button. It will close the 'Add new target' dialog box and You'll see that a line appears to the box left from button box.\n"
    .getstart.text insert end "\n8. You see two triangles upside the box where did appear the line. Click a left mouse button on the triangle, hold the button down and move the mouse. See what happens. Play a little around with these triangles and You understand why are these there.\n"
    .getstart.text insert end "\n9. Click on that line with left mouse button.\n"
    .getstart.text insert end "\n10. Press the button 'Make' in the button box. It will execute a command 'make -f <makefile> <target>' where <makefile> is a makefile You specified and <target> is a target You specified.\n"
    .getstart.text insert end "\n11. Add some more target descriptions in the way described above.\n"
    .getstart.text insert end "\n12. Press the button 'Save as' to save the target listing You described. A file selection dialog box does appear. Choose a file name and press the 'Save' button in it. You can use the target listing later by pressing 'Open' button in button box. Then file selection dialog box appears again. Just choose the file where the listing is saved and press the open button in it.\n"
    .getstart.text insert end "\n13. Have a good work with MaMa!\n"

    .getstart.text insert end "\nRead the MaMa's tutorial manual for more information about MaMa.\n"

    .getstart.text configure -state disabled
    focus .getstart.scroll

    bind .getstart       <Escape> {destroy .getstart}
    bind .getstart.close <Return> {destroy .getstart}
}

#
# ShowTutorial --
# display 'Tutorial' text.
#
proc ShowTutorial {} {
    global HelpSectionFont HelpTextFont HelpTitleFont

    catch {destroy .tutorial}
    toplevel .tutorial
    wm title .tutorial "MaMa Tutorial"
    wm iconname .tutorial "Tutorial"
    positionWindow .tutorial 100 10

    text      .tutorial.text -yscroll ".tutorial.scroll set" -wrap word \
	    -width 60 -height 30 -font $HelpTextFont -relief sunken \
	    -takefocus 0 -setgrid 1
    scrollbar .tutorial.scroll -orient vertical \
	    -command ".tutorial.text yview" -highlightthickness 0
    button    .tutorial.close -text Close -command "destroy .tutorial"

    pack .tutorial.close  -side bottom -fill x
    pack .tutorial.scroll -side right -fill y
    pack .tutorial.text   -expand yes -fill both

    .tutorial.text tag configure title   -font $HelpTitleFont
    .tutorial.text tag configure section -font $HelpSectionFont

    .tutorial.text insert end "\n    Tutorial Manual about MaMa\n" title

    .tutorial.text insert end "\n\n1. Make Program and Makefiles\n" section
    .tutorial.text insert end "\nBecause of I've not a make program author I try to quote some pieces from manual page, by Dennis Morse, of GNU Make version 3.76.1, by Richard Stallman and Roland McGrath, instead of thinkink the text out myself. I quote here only pieces because I assume You have an access to make utility manual pages and You are already familiar with the utility. Still I think it should be good here to give some reference words about make program itself too for those who still doesn't know yet what is make.\n"
    .tutorial.text insert end "\nThe purpose of the make utility is to determine automatically which pieces of a large program need to be recompiled, and issue the commands to recompile them. It is most commonly used on compiling programs, especially C programs, but in fact, make is not limited to programs. You can use it to describe any task where some files must be updated automatically from others whenever the others change.\n"
    .tutorial.text insert end "\nTo prepare to use make, you must write a file called the makefile that describes the relationships among files in your program, and the states the commands for updating each file. In a program, typically the executable file is updated from object files, which are in turn made by compiling source files. Once a suitable makefile exists, each time you change some source files, this simple shell command:\n"
    .tutorial.text insert end "\n\tmake\n"
    .tutorial.text insert end "\nsuffices to perform  all  necessary  recompilations.\n"
    .tutorial.text insert end "\nFor more information on how make program determines the files to be updated and how to write makefiles, see make program manual pages that are included to the make program release You are using.\n"

    .tutorial.text insert end "\n\n2. General Layout\n" section
    .tutorial.text insert end "\nMaMa general layout consists of four sections on screen. These are following.\n\n1. Menu bar at the top of MaMa layout. It consists of several menu buttons that contain all commands and functions MaMa can perform.\n\n2. Target list under the menu bar. Target list displays a listing of targets in makefiles and comments describing the targets. These are shown in table form: makefile name in first column, target name in second column and comment in third column, as seen by the titles of columns displayed over the listing.\n\n3. Button box to the right from target listing. It consists of buttons for most usefull commands to perform operations on selected line in target listing.\n\n4. Output box under the target listing and button box. The output of executed 'make' command is displayed there.\n"

    .tutorial.text insert end "\n\n3. Adding New Targets to Target List\n" \
	    section
    .tutorial.text insert end "\nTo make use of MaMa there must be something in target listing. One line in target list describes a single 'make' command. Here is described step by step how to add a line to target listing.\n"
    .tutorial.text insert end "\n1. Press the 'Add' button on button box or select 'Add' command from 'Edit' menu at menu bar or hit Ctrl+a. All these three are equal. You see a new window appearing. It is a form for entering information about target and should be quite understandable for You as a programmer or any computer specialist. If You want to cancel the operation of entering target specification, press the 'Cancel' button.\n"
    .tutorial.text insert end "\n2. Enter the makefile name to the first line and if needed, include a relative or absolute path to it. You can browse a directory for searching makefile by pressing 'Browse' button at the end of the line -- a typical file browsing dialog box appears. Select a file and click on 'Open' button or press 'Return' key on Your keyboard. File selection dialog box disappears and the selected makefile appears into the first line.\n"
    .tutorial.text insert end "\n3. Specify an existing target in the selected makefile on the second line. You have to know the target name on Your own. MaMa doesn't check if Your entered target exists at all in that makefile. You may leave it blank. Then executing 'make' program, 'make' doesnt't get the target name and it completes the first target found in the makefile by default.\n"
    .tutorial.text insert end "\n4. Add a comment to the third line. That should describe with few words the nature of this target in the makefile. You may leave it blank too. The only purpose of that line is to explain for You as a humand being the nature of the target with two or three words. So, if You feel You know the meaning of the target anyway, You don't need the comment at all.\n"
    .tutorial.text insert end "\n5. Press an 'OK' button. It will close the new target adding dialog box and You'll see that a line appears to the target list box left from button box.\n"
    .tutorial.text insert end "\nInstead of clicking 'Add' button You can also insert a line by clicking 'Insert' button on buttonbox. As in case of 'Add' command the equal of 'Insert' button pressing are selecting 'Insert' command from 'Edit' menu or hitting Ctrl+i. The difference between 'Add' and 'Insert' commands come out if there are at least one line in target listing before using these commands. You can select a line in target listing by clicking left mouse button on the line. 'Add' command iserts a new line to target listing next to the selected line. 'Insert' command inserts a new line to target listing right before selected line. If there is no line selected, 'Add' command appends a new line to the end of listing and 'Insert' command appends a new line to the beginning of listing.\n"

    .tutorial.text insert end "\n\n4. Executing a 'make' command\n" section
    .tutorial.text insert end "\nFor executing a 'make' command there must be at least one line in target list. See section 3 about inserting lines into target listing.\n\nAs said before (in section 3.), one line in target list decribes a single 'make' command. The form of the command that MaMa will execute if You want it to do it is 'make -f <makefile> <target>' where <makefile> is a makefile You specified and <target> is a target You specified. For executing a 'make' command select a line in target listing by click on the line at first. Then select a 'Make' command from 'Run' menu or press the button 'Make' on button box or hit 'Return' key. It executes a 'make' command described above. You can see an output of 'make' in Output box under the target listing and button box.\n"

    .tutorial.text insert end \
	    "\n\n5. Editing Target List and Targets in It\n" section
    .tutorial.text insert end "\nThe size of columns in target list box can be changed by ruler bar over the column titles. There are two triangles on the ruler bar. Leftmost triangle separates first column from second and rightmost triangle separates second column from third one. The borders between columns can be changed by moving these triangles accordingly.\n"
    .tutorial.text insert end "\nYou can change the order of lines in listing. To do this select the line that You want to move up or down by clicking on it with mouse at first. If You want to move the line up, select a command 'Up' from 'Edit' menu or click the button 'Up' in button box or hit Ctrl+<arrow up key>. All these three are equal. If You want to move the line down, select a command 'Down' from 'Edit' menu or click the button 'Down' in button box or hit Ctrl+<arrow down key>. All these three are equal.\n"
    .tutorial.text insert end "\nIf You selected a line in target listing You may want to change the selection. You may do it by clicking a mouse on another line You want to select or use arrow keys. Hitting up or down arrow key the line before or after (accordingly) selected line will be selected instead.\n"
    .tutorial.text insert end "\nYou can change the condition of line in target listin. To do this, select the line by clicking on it with mouse at first. Then select a command 'Edit' from 'Edit' menu or press an 'Edit' button in button box or hit Ctrl+e. All these three are equal. You see a dialog box appearing that is described in adding a line into target list too (see section 3.). Change the three lines in that form according to Your needs and click 'OK' button. Condition of the line in target listing is now changed."

    .tutorial.text insert end "\n\n6. Saving and Opening Target List\n" section
    .tutorial.text insert end "\nYou can save the target listing You are working with for another time You want to work with it again. If You are doing this first time, select a command 'Save' or 'Save as ...' from 'File' menu or Click the button 'Save' or 'Save as' in button box or hit Ctrl+s or Ctrl+S. All these are equal. A typical file browsing dialog box appears. Choose a file or enter a new file name You want to save the list to and click the 'Save' button or hit 'Return' key. The listing is now saved to the file You specified.\n"
    .tutorial.text insert end "\nTo load the listing from file You saved it to, choose a command 'Open' from 'File' menu or press the button 'Open' in button box or hit Ctrl+o. All these are equal. A typical file browsing dialog box appears. Choose file You saved it to and click 'Open' or hit 'Return' key. The target listing You saved last time to this file appears now to target list box.\n"
    .tutorial.text insert end "\nIf You have loaded a target listing from a file or have saved it already during the current work, 'Save' and 'Save as ...' commands are different. 'Save' command saves the target listing to the same file You saved it before or loaded it from without asking a filename from You. 'Save as ...' will act always the same way as described at the beginning of this section.\n"
    .tutorial.text insert end "\nIf You are working with a target listing and You want to create a new target listing, choose a command 'New' from 'File' menu or press the button 'New' in button box or hit Ctrl+n. All these are equal. Then, if You have any unsaved changes in current target listing, MaMa asks You if You want to save the current listing and then it clears the target listing box. Now You can start to build a new target listing as described before.\n"

    .tutorial.text insert end "\n\n7. Keyboard control\n" section
    .tutorial.text insert end "\nIt is possible to do almost everything in MaMa by using a keyboard only. As described before, all commands to MaMa has their keyboard equivalents. If You don't remember the key combination for invoking the command, You can see it in menu where the command is -- each command has an according key combination after the name of the command. If You want to select some line from target listing box, You can use <Tab> key to move the focus to the target listing box before You can use arrow keys. You see the focus is on target list box if there is a black lined box appearing around the target list box. Each menu on menubar opens with Alt+<underlined character on the menu button>. For example 'File' menu opens with Alt+F.\n"
    .tutorial.text insert end "\nThe only things You need mouse for are the dialog boxes that ask You for 'Yes' or 'No' for something. If You want to answer to these Yes/No questions something that isn't a defaul answer, You have to use mouse.\n"

    .tutorial.text insert end "\n\n8. Target List File Structure\n" section
    .tutorial.text insert end "\nIt is possible to edit the contents of saved MaMa target listing file. Maybe it is needed if the file is corrupted somehow or something. However, while there is no need for doing so, it is not succested.\n"
    .tutorial.text insert end "\nEach line in the listing is on a separate line in that file. Lines that contain nothing (only whitespaces) are ignored. All other lines are proceeded. The first word on the line is makefile specification, second is target and remaining string is considered as a contents of comment field in target description.\n"
    .tutorial.text insert end "\nSpaces at the beginning of line (You probably added the spaces manually) are ignored and will be disappeared if You save the listing from MaMa. The firs space after makefile name is followed by target name and second space is followed by comment, so, if no target name is specified and comment must be specified, there must be still two spaces between makefile name and target comment text. Additional spaces in comment text are not removed. Limitation is that makefile name and target name may not contain spaces (anyway, I havent seen someone really needs it -- if You need, let me know :)).\n"

    .tutorial.text insert end "\n\n9. Configuration of MaMa\n" section
    .tutorial.text insert end "\nThe current version, as the first version, of MaMa is totally not flexible. The only thing You can specify as a setting of MaMa, is an initial MaMa target listing file that has to be loaded right after running MaMa. Unfortunately You have to do it manually via .mamarc file at Your home directory. There is no graphical interface for that yet. To do it You have to enter a line 'set MAMAFILE yourmamafile.mam' to the .mamarc file.\n"
    .tutorial.text insert end "\nI try to add a lot of flexibility to MaMa as soon as possible e.g. as soon as I find some will and spare time for that. If You don't like a condition of MaMa and You are a Tcl/Tk guru, You may look at the MaMa's Tcl/Tk code and figure out how to change something You want to and change it accordingly or wait for next releases that will be more flexible than this one.\n"
    .tutorial.text insert end "\nWhy I didn't add any flexibility to this version? It is because of it is the first version. I haven't used the tool myself yet so much to understand and decide what is really reasonable to make flexibel at the first place. Give me a little time to get used to this utility and to get really annoyed of some features of MaMa. Then I can better understand what is bad and what is better -- I hope I think quite the same way as You because I'm a programmer too.\n"
    .tutorial.text insert end "\nStill You too can help and speed up the development process by trying to use it on Your makefiles. I'll appreciate Your help!\n"

    .tutorial.text insert end "\n\n10. Getting Help about MaMa\n" section
    .tutorial.text insert end "\nTo get help about anything in mama, see the 'Introduction' or 'Controls' in help menu. 'Introduction' describes briefly all the labels in help menu and 'Controls' does it more briefly.\n"

    .tutorial.text configure -state disabled
    focus .tutorial.scroll

    bind .tutorial       <Escape> {destroy .tutorial}
    bind .tutorial.close <Return> {destroy .tutorial}
}

#
# ShowControls --
# display 'Controls' text.
#
proc ShowControls {} {
    global HelpTextFont HelpTitleFont

    catch {destroy .controls}
    toplevel .controls
    wm title .controls "MaMa Controls"
    wm iconname .controls "Controls"
    positionWindow .controls 100 10

    text      .controls.text -yscroll ".controls.scroll set" -wrap word \
	    -width 60 -height 30 -font $HelpTextFont -relief sunken \
	    -takefocus 0 -setgrid 1
    scrollbar .controls.scroll -orient vertical \
	    -command ".controls.text yview" -highlightthickness 0
    button    .controls.close -text Close -command "destroy .controls"

    pack .controls.close  -side bottom -fill x
    pack .controls.scroll -side right -fill y
    pack .controls.text   -expand yes -fill both

    .controls.text tag configure title     -font $HelpTitleFont

    .controls.text insert end "\n    MaMa Controls\n" title

    .controls.text insert end "\nEach menu opens with Alt+<underlined character on the menu button>. For example 'File' menu opens with Alt+F.\n"

    .controls.text insert end "\n\t\t     Key"
    .controls.text insert end "\nCommand\tcombination\tButton\tComment\n"

    .controls.text insert end \
	    "\nEdit->Add\tCtrl+a\t\tAdd\tAdd target to listing"
    .controls.text insert end \
	    "\nEdit->Down\tCtrl+<Down>\tDown\tLine move down 1 step"
    .controls.text insert end \
	    "\nEdit->Edit\tCtrl+e\t\tEdit\tAlter selected target"
    .controls.text insert end \
	    "\nEdit->Insert\tCtri+i\t\tInsert\tInsert target to listing"
    .controls.text insert end \
	    "\nEdit->Up\t\tCtrl+<Up>\tUp\tLine move up 1 step\n"

    .controls.text insert end \
	    "\nFile->Exit\t\tEsc\t\tExit\tExit from MaMa"
    .controls.text insert end \
	    "\nFile->New\tCtrl+n\t\tNew\tCreate a new listing"
    .controls.text insert end \
	    "\nFile->Open\tCtrl+o\t\tOpen\tListing from file"
    .controls.text insert end \
	    "\nFile->Save\tCtrl+s\t\tSave\tSave listing"
    .controls.text insert end \
	    "\nFile->Save as ...\tCtrl+S\t\tSave as\tSave listing as ...\n"

    .controls.text insert end \
	    "\nHelp->About\tCtrl+A\t\t\tShow 'About' message"
    .controls.text insert end \
	    "\nHelp->Controls\tCtrl+C\t\t\tMaMa controls listing"
    .controls.text insert end \
	    "\nHelp->Done\tCtrl+D\t\t\tDone works in MaMa"
    .controls.text insert end \
	    "\nHelp->Get Started\tCtrl+R\t\t\tGuide to getting started"
    .controls.text insert end \
	    "\nHelp->Ideas\tCtrl+E\t\t\tShow Ideas listing"
    .controls.text insert end \
	    "\nHelp->Introduction\tCtrl+I\t\t\tIntroduction to MaMa"
    .controls.text insert end \
	    "\nHelp->Known Pr.\tCtrl+K\t\t\tKnown Problems listing"
    .controls.text insert end \
	    "\nHelp->Support\tCtrl+P\t\t\tSupport info"
    .controls.text insert end \
	    "\nHelp->Tutorial\tCtrl+T\t\t\tMaMa's tutorial manual\n"

    .controls.text insert end \
	    "\nRun->Make\tReturn\t\tMake\tExecute make program\n"

    .controls.text configure -state disabled
    focus .controls.scroll

    bind .controls       <Escape> {destroy .controls}
    bind .controls.close <Return> {destroy .controls}
}

#
# ShowDone --
# display 'Done' text.
#
proc ShowDone {} {
    global HelpSectionFont HelpTextFont HelpTitleFont

    catch {destroy .done}
    toplevel .done
    wm title .done "Done Works in MaMa"
    wm iconname .done "Done"
    positionWindow .done 100 10

    text      .done.text -yscroll ".done.scroll set" -wrap word \
	    -width 60 -height 30 -font $HelpTextFont -relief sunken \
	    -takefocus 0 -setgrid 1
    scrollbar .done.scroll -orient vertical \
	    -command ".done.text yview" -highlightthickness 0
    button    .done.close -text Close -command "destroy .done"

    pack .done.close  -side bottom -fill x
    pack .done.scroll -side right -fill y
    pack .done.text   -expand yes -fill both

    .done.text tag configure title   -font $HelpTitleFont
    .done.text tag configure section -font $HelpSectionFont

    .done.text insert end "\n    Done Works in MaMa\n" title

    .done.text insert end "\n\nVersion 1.0b0\n" section
    .done.text insert end "\n* Fixed the nasty problem with Tcl special characters in target listing description (both in active target list and in save files). While fixing the bug, save file format has changed a bit (see the section 'Target List File Structure', if interested in manual editing)\n"
    .done.text insert end "\n* Lines in target listing file that begins with spaces (You probably added the spaces manually) don't disappear anymore -- the spaces are ignored.\n"
    .done.text insert end "\n* The exit status of make program is added now at the end of the last line in the box that displays make's output.\n"
    .done.text insert end "\n* Fixed the bug that generated make program's error if target name missed in target description.\n"
    .done.text insert end "\n* MaMa doesn't let anymore to run another making job while running one already. It is because possibly invoking another job at the same time happens accidentally. If the situation will be needed still, maybe I'll add some better solution for handling it.\n"
    .done.text insert end "\n\nVersion 1.0a3\n" section
    .done.text insert end "\n* Fixed the problem with 'make' program if 'make' uses not sh while installing MaMa.\n"
    .done.text insert end "\n\nVersion 1.0a2\n" section
    .done.text insert end "\n* Added an IRIX port to setup.sh. Tested on 6.2. Thanx to Ants Aader for sacrificing his time for it.\n"
    .done.text insert end "\n* Made some conventions to setup.sh to deal with porting issues.\n"
    .done.text insert end "\n* Fixed bug -- 'make' program pipes it's output to MaMa. It wasn't closed correctly. Now it is ok.\n"
    .done.text insert end "\n* 'make clean' in src directory of MaMa doesn't remove mama.rc any more.\n"
    .done.text insert end "\n\nVersion 1.0a1\n" section
    .done.text insert end "\n* Fixed a problem occured with setup program while bash missed -- made the script compatible with sh.\n"
    .done.text insert end "\n* Added a note to the end of setup program execution about older version executables and libraries not to be deleted.\n"
    .done.text insert end "\n* Some preening in MaMa package files.\n"

    .done.text insert end "\n\nVersion 1.0a0\n" section
    .done.text insert end "\nImplemented driver with graphical user interface for make program.\n"

    .done.text configure -state disabled
    focus .done.scroll

    bind .done       <Escape> {destroy .done}
    bind .done.close <Return> {destroy .done}
}

#
# ShowIdeas --
# display 'Ideas' text.
#
proc ShowIdeas {} {
    global HelpTextFont HelpTitleFont

    catch {destroy .ideas}
    toplevel .ideas
    wm title .ideas "Ideas Related to MaMa"
    wm iconname .ideas "Ideas"
    positionWindow .ideas 100 10

    text      .ideas.text -yscroll ".ideas.scroll set" -wrap word \
	    -width 60 -height 30 -font $HelpTextFont -relief sunken \
	    -takefocus 0 -setgrid 1
    scrollbar .ideas.scroll -orient vertical \
	    -command ".ideas.text yview" -highlightthickness 0
    button    .ideas.close -text Close -command "destroy .ideas"

    pack .ideas.close  -side bottom -fill x
    pack .ideas.scroll -side right -fill y
    pack .ideas.text   -expand yes -fill both

    .ideas.text tag configure title   -font $HelpTitleFont

    .ideas.text insert end "\n    Ideas Related to MaMa\n" title

    .ideas.text insert end "\nNOTE: the ideas here may not be realistic. These are just the ideas I have seen once in my dreams or thought about sometimes while driving along the long ways by bus e.t.c. The only thing that I can say in realistic condition is that my goal is to make an utility for programmers. It means that maybe someday MaMa goes beyond the manipulations of 'make' program and makefiles -- for example starts to be able to backup the directories holding software that has to be developed. Word 'MaMa' is shorter version of words 'Make Master' and it is 'Make Master' only because of basic idea that wast to add a graphical user interface for make program. Ideas described below may not be realistic also because of I build the utility from my spare time. I.e. guite likely things here are evolving slowly and maybe I don't have any time to deal it with in future at all... Nevertheless let's hope the best and You can provide me with more or less realistic ideas anytime!\n"
    .ideas.text insert end "\nHere comes the listing of ideas.\n"
    .ideas.text insert end "\n* Implement a convenient makefile generator.\n"
    .ideas.text insert end "\n* Add a possibility to edit targets in makefile separately.\n"
    .ideas.text insert end "\n* Add a graphical interface written in Tcl/Tk too to setup program. Why not to rewrite a not graphical part of setup in tcl too?\n"
    .ideas.text insert end "\n* Writing help texts should be more convenient for myself... So, I have some time to spend on making some simple tools handling help pages. I didn't want to do it at the _beginning_ because It is possible to get a better idea what tools are really needed _after_ trying to do something without any special tools. So, now I have a clear picture in my mind what I need.\n"
    .ideas.text insert end "\n* Add some kind of navigation system to help: hyperlinks, 'back' and 'forward' buttons, searching...\n"
    .ideas.text insert end "\n* As a result of last to ideas the help system maybe usable somewhere else, not only at MaMa.\n"
    .ideas.text insert end "\n* Programmers like to backup their work. So, why not to see a button 'Backup' together with a button 'Unbackup' somewhere on screen that does all the work (tar, gzip, whatever) if clicking on that? Idea -- add it to MaMa.\n"
    .ideas.text insert end "\n* Is it possible to integrate MaMa with 'emacs' using 'emacs client' so, it is possible to parse the C compiler error messages in the way emacs it does? I haven't tried it yet -- just an idea. If possible, why not to try to do it. Wouldn't it be convenient for people using emacs for programming (like me)?\n"
    .ideas.text insert end "\n* A good published software has a good installation or setup tool with the published software package. Think about possibilities of adding features to MaMa that makes such a tool creation more convenient. Also a good install program has an option to deinstall the installed software. Voice in my stomach echos to me that it should be possible to generate the option automatically. So, let's teach it to MaMa in future... maybe...\n"
    .ideas.text insert end "\n* Design an integrated environment for setup program so, there is no need to keep a separate setup program for each package that is provided with the setup program as an utility for deinstallation after installation.\n"
    .ideas.text insert end "\n* Fix all bugs in Mama.\n"
    .ideas.text insert end "\n* ...\n"

    .ideas.text configure -state disabled
    focus .ideas.scroll

    bind .ideas       <Escape> {destroy .ideas}
    bind .ideas.close <Return> {destroy .ideas}
}

#
# ShowKnownProblems --
# display 'Known Problems' text.
#
proc ShowKnownProblems {} {
    global HelpTextFont HelpTitleFont

    catch {destroy .bugs}
    toplevel .bugs
    wm title .bugs "Known Problems Related to MaMa"
    wm iconname .bugs "Problems"
    positionWindow .bugs 100 10

    text      .bugs.text -yscroll ".bugs.scroll set" -wrap word \
	    -width 60 -height 30 -font $HelpTextFont -relief sunken \
	    -takefocus 0 -setgrid 1
    scrollbar .bugs.scroll -orient vertical \
	    -command ".bugs.text yview" -highlightthickness 0
    button    .bugs.close -text Close -command "destroy .bugs"

    pack .bugs.close  -side bottom -fill x
    pack .bugs.scroll -side right -fill y
    pack .bugs.text   -expand yes -fill both

    .bugs.text tag configure title   -font $HelpTitleFont

    .bugs.text insert end "\n    Known Problems Related to MaMa\n" title

    .bugs.text insert end "\nNOTE: Here are described the listing of problems or bugs related to MaMa's current version, that I already know. So, You don't have to report me if You encounter some of these ones while working with MaMa, except You know how to solve it and You want to tell me how to solve it. If You encounter a problem that isn't described here, get the latest version or see it's latest change log, and see, if the problem is fixed already. You can do both at http://www.cs.ut.ee/~asko/mama/. If there's no information about the problem, mail it to asko@ut.ee, please.\n"

    .bugs.text insert end "\nHere comes the listing of known problems.\n"

    .bugs.text insert end "\n* The targets listing loaded from files saved with older versions of MaMa, than 1.0b0, contain '{}' at 'Description' field if there wasn't any description formerly. SOLUTION: Edit the targets, delete {} at 'Description' field and save the listing. '{}' should not appear any more. REASON: It isn't a bug. It is because of bugfix related to tcl special characters -- save file format has changed.\n"
    .bugs.text insert end "\n* There are 'Yes/No' (for example 'Exit' verification) type of dialogboxes in MaMa that aren't completely possible to handle from keyboard. The reason is that I use tk_dialog, the Tcl/Tk built-in command, for that and it enables only default answer to select from keyboard. SOLUTION: Use mouse. FIX: Looks like I have to write my own dialog in future.\n"
    .bugs.text insert end "\n* Looks like Control-T has some magical binding. I don't understand which one. Just it behaves very strangely if Control-T is pressed. Because that feature the popup of tutorial help isn't binded to Control-T but Control-R. SOLUTION: Try to not press Control-T. If You press, MaMa hangs for some seconds. Nothing more special doesn't happen (at least I haven't noticed). FIX: Just now I don't know yet how to fix it.\n"

    .bugs.text configure -state disabled
    focus .bugs.scroll

    bind .bugs       <Escape> {destroy .bugs}
    bind .bugs.close <Return> {destroy .bugs}
}

#
# ShowSupport --
# display 'Support' text.
#
proc ShowSupport {} {
    global HelpTextFont HelpTitleFont

    catch {destroy .support}
    toplevel .support
    wm title .support "Support for MaMa Users"
    wm iconname .support "Support"
    positionWindow .support 100 10

    text      .support.text -yscroll ".support.scroll set" -wrap word \
	    -width 60 -height 30 -font $HelpTextFont -relief sunken \
	    -takefocus 0 -setgrid 1
    scrollbar .support.scroll -orient vertical \
	    -command ".support.text yview" -highlightthickness 0
    button    .support.close -text Close -command "destroy .support"

    pack .support.close  -side bottom -fill x
    pack .support.scroll -side right -fill y
    pack .support.text   -expand yes -fill both

    .support.text tag configure title   -font $HelpTitleFont

    .support.text insert end "\n    Support for MaMa Users\n" title

    .support.text insert end "\nIf You encounter any problem or bug in using MaMa (like MaMa doesn't listen You or something) and it is not described in 'Help->Known Problems' feel free to send an email to me (asko@ut.ee) and ask me for help if need any. If You don't need a help, You can still send an email and a solution with it (if You know one and want to share it with me) to me because of maybe I don't know the problem still and then I can do something for fixing it in next release.\n"

    .support.text insert end "\nAny other questions, ideas and 'keep-going-suggestions' about MaMa are also welcome. I'll appreciate any kind of help.\n"

    .support.text insert end "\nThere is also a WWW page for MaMa available:\n"
    .support.text insert end "\n\thttp://www.cs.ut.ee/~asko/mama/\n"

    .support.text insert end "\nNOTE: Because of i deal with implementing MaMa from my spare time, I have to write a diploma work and I have a job at the same time, the answers to Your email letters may take some time, but if something weird will not happen, You should get an answer.\n"

    .support.text configure -state disabled
    focus .support.scroll

    bind .support       <Escape> {destroy .support}
    bind .support.close <Return> {destroy .support}
}
